@@ -31,6 +31,7 @@ import com.mikepenz.iconics.IconicsDrawable;
 import com.mikepenz.iconics.IconicsSize;
 import com.mikepenz.iconics.typeface.IIcon;
 
+import net.programmierecke.radiodroid2.players.PlayStationTask;
 import net.programmierecke.radiodroid2.players.selector.PlayerSelectorDialog;
 import net.programmierecke.radiodroid2.players.selector.PlayerType;
 import net.programmierecke.radiodroid2.service.ConnectivityChecker;
@@ -53,9 +54,12 @@ import java.net.Proxy;
 import java.security.KeyStore;
 import java.util.ArrayList;
 import java.util.Date;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.TrustManager;
@@ -276,6 +280,27 @@ public class Utils {
         return null;
     }
 
+    public static List<DataRadioStation> getStationsByUuid(OkHttpClient httpClient, Context ctx, Iterable<String> listUUids) {
+        String uuids = TextUtils.join(",", listUUids);
+        Log.d("UTIL", "Search by uuid for items");
+        HashMap<String, String> p = new HashMap<String, String>();
+        p.put("uuids", uuids);
+        String result = Utils.downloadFeedRelative(httpClient, ctx, "json/stations/byuuid", true, p);
+        if (result != null) {
+            try {
+                List<DataRadioStation> list = DataRadioStation.DecodeJson(result);
+                if (list != null) {
+                    return list;
+                }else{
+                    Log.e("UTIL", "stations by uuid was null");
+                }
+            } catch (Exception e) {
+                Log.e("UTIL", "getStationsByUuid() " + e);
+            }
+        }
+        return null;
+    }
+
     public static @Nullable
     DataRadioStation getCurrentOrLastStation(@NonNull Context ctx) {
         DataRadioStation station = PlayerServiceUtil.getCurrentStation();
@@ -300,9 +325,19 @@ public class Utils {
 
     public static void showPlaySelection(final RadioDroidApp radioDroidApp, final DataRadioStation station, final FragmentManager fragmentManager) {
         SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(radioDroidApp);
-        final boolean play_external = sharedPref.getBoolean("play_external", false);
+        final boolean externalAvailable = sharedPref.getBoolean("play_external", false);
 
-        if (radioDroidApp.getMpdClient().isMpdEnabled() || play_external || CastHandler.isCastSessionAvailable()) {
+        CastHandler castHandler = radioDroidApp.getCastHandler();
+        final boolean castAvailable = castHandler.isCastSessionAvailable();
+
+        final boolean mpdAvailable = radioDroidApp.getMpdClient().isMpdEnabled();
+
+        if (castAvailable && !externalAvailable && !mpdAvailable) {
+            new PlayStationTask(station, radioDroidApp.getApplicationContext(),
+                    url -> castHandler.playRemote(station.Name, url, station.IconUrl),
+                    null)
+                    .execute();
+        } else if (externalAvailable || mpdAvailable) {
             showMpdServersDialog(radioDroidApp, fragmentManager, station);
         } else {
             playAndWarnIfMetered(radioDroidApp, station, PlayerType.RADIODROID, () -> play(radioDroidApp, station));
@@ -319,6 +354,11 @@ public class Utils {
                 });
     }
 
+    public static boolean urlIndicatesHlsStream(String streamUrl) {
+        final Pattern p = Pattern.compile(".*\\.m3u8([#?\\s].*)?$");
+        return p.matcher(streamUrl).matches();
+    }
+
     public interface MeteredWarningCallback {
         void warn(DataRadioStation station, PlayerType playerType);
     }
@@ -372,6 +412,10 @@ public class Utils {
             return R.style.MyMaterialTheme;
     }
 
+    public static boolean isDarkTheme(final Context context) {
+        return getThemeResId(context) == R.style.MyMaterialTheme_Dark;
+    }
+
     public static int getTimePickerThemeResId(final Context context) {
         int theme;
         if (getThemeResId(context) == R.style.MyMaterialTheme_Dark)
